"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lib_reflect_1 = require("lib-reflect");
const lodash_1 = require("lodash");
var InterceptorType;
(function (InterceptorType) {
    InterceptorType[InterceptorType["Before"] = 1] = "Before";
    InterceptorType[InterceptorType["After"] = 2] = "After";
})(InterceptorType = exports.InterceptorType || (exports.InterceptorType = {}));
class InterceptorData {
    constructor() {
        this.type = InterceptorType.Before | InterceptorType.After;
    }
    createInstance() {
        if (!lodash_1.isNil(this.instance)) {
            return;
        }
        if (lodash_1.isFunction(this.target)) {
            this.instance = Reflect.construct(this.target, []);
        }
        else {
            this.instance = this.target;
        }
    }
}
exports.InterceptorData = InterceptorData;
exports.UseActivation = (activation, type) => lib_reflect_1.AnyDecoratorFactory((classData, methodOrProp) => {
    const data = new InterceptorData();
    data.target = activation;
    if (!lodash_1.isNil(type)) {
        data.type = type;
    }
    if (lodash_1.isNil(methodOrProp)) {
        // Class decorator
        classData.attributesData.push(data);
    }
    else if (lib_reflect_1.checkIfInstanceOf(methodOrProp, lib_reflect_1.MethodData)) {
        // Method decorator
        methodOrProp.attributesData.push(data);
    }
    else {
        throw new Error('Invalid usage of the UseActivation decorator.');
    }
});
exports.UseBefore = (activation) => exports.UseActivation(activation, InterceptorType.Before);
exports.UseAfter = (activation) => exports.UseActivation(activation, InterceptorType.After);
//# sourceMappingURL=use.activation.js.map